@@ -89,13 +89,13 @@ class S3FileSystem(object):
         uses the key/secret given, or boto's credential resolver (environment
         variables, config files, EC2 IAM server, in that order)
     key : string (None)
-        If not anonymouns, use this key, if specified
+        If not anonymous, use this key, if specified
     secret : string (None)
         If not anonymous, use this password, if specified
     use_ssl : bool (True)
         Whether to use SSL in connections to S3; may be faster without, but
         insecure
-    client_kwargs : dict of paramaters for the boto3 client
+    client_kwargs : dict of parameters for the boto3 client
     requester_pays : bool (False)
         If RequesterPays buckets are supported.
     default_block_size: None, int
@@ -194,7 +194,7 @@ class S3FileSystem(object):
         return self._conn[tok]
 
     def get_delegated_s3pars(self, exp=3600):
-        """Get temporary credentials from STS, apropriate for sending across a
+        """Get temporary credentials from STS, appropriate for sending across a
         network. Only relevant where the key/secret were explicitly provided.
 
         Parameters
@@ -364,6 +364,77 @@ class S3FileSystem(object):
             except (ClientError, ParamValidationError):
                 raise FileNotFoundError(path)
 
+    _metadata_cache = {}
+
+    def metadata(self, path, refresh=False):
+        """ Return metadata of path.
+
+        Metadata is cached unless `refresh=True`.
+
+        Parameters
+        ----------
+        path : string/bytes
+            filename to get metadata for
+        refresh : bool (=False)
+            if False, look in local cache for file metadata first
+        """
+        bucket, key = split_path(path)
+
+        if refresh or path not in self._metadata_cache:
+            response = self.s3.head_object(Bucket=bucket,
+                                           Key=key,
+                                           **self.req_kw)
+            self._metadata_cache[path] = response['Metadata']
+
+        return self._metadata_cache[path]
+
+    def getxattr(self, path, attr_name):
+        """ Get an attribute from the metadata.
+
+        Examples
+        --------
+        >>> mys3fs.getxattr('mykey', 'attribute_1')  # doctest: +SKIP
+        'value_1'
+        """
+        xattr = self.metadata(path)
+        if attr_name in xattr:
+            return xattr[attr_name]
+        return None
+
+    def setxattr(self, path, **kw_args):
+        """ Set metadata.
+
+        Attributes have to be of the form documented in the `Metadata Reference`_.
+
+        Parameters: key-value pairs like field="value", where the values must be strings. Does not alter existing fields, unless
+        the field appears here - if the value is None, delete the field.
+
+        Examples
+        --------
+        >>> mys3file.setxattr(attribute_1='value1', attribute_2='value2')  # doctest: +SKIP
+
+        .. Metadata Reference:
+        http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingMetadata.html#object-metadata
+        """
+
+        bucket, key = split_path(path)
+        metadata = self.metadata(path)
+        metadata.update(**kw_args)
+
+        # remove all keys that are None
+        for kw_key in kw_args:
+            if kw_args[kw_key] is None:
+                metadata.pop(kw_key, None)
+
+        self.s3.copy_object(CopySource="{}/{}".format(bucket, key),
+                            Bucket=bucket,
+                            Key=key,
+                            Metadata=metadata,
+                            MetadataDirective='REPLACE')
+
+        # refresh metadata
+        self._metadata_cache[path] = metadata
+
     def _walk(self, path, refresh=False):
         if path.startswith('s3://'):
             path = path[len('s3://'):]
@@ -601,6 +672,7 @@ class S3FileSystem(object):
         if not self.exists(path):
             raise FileNotFoundError(path)
         if recursive:
+            self.invalidate_cache(path)
             self.bulk_delete(self.walk(path))
             if not self.exists(path):
                 return
@@ -788,6 +860,37 @@ class S3File(object):
         """ File information about this path """
         return self.s3.info(self.path)
 
+    def metadata(self, refresh=False):
+        """ Return metadata of file.
+        See :func:`~s3fs.S3Filesystem.metadata`.
+
+        Metadata is cached unless `refresh=True`.
+        """
+        return self.s3.metadata(self.path, refresh)
+
+    def getxattr(self, xattr_name):
+        """ Get an attribute from the metadata.
+        See :func:`~s3fs.S3Filesystem.getxattr`.
+
+        Examples
+        --------
+        >>> mys3file.getxattr('attribute_1')  # doctest: +SKIP
+        'value_1'
+        """
+        return self.s3.getxattr(self.path, xattr_name)
+
+    def setxattr(self, **kwargs):
+        """ Set metadata.
+        See :func:`~s3fs.S3Filesystem.setxattr`.
+
+        Examples
+        --------
+        >>> mys3file.setxattr(attribute_1='value1', attribute_2='value2')  # doctest: +SKIP
+        """
+        if self.mode != 'rb':
+            raise NotImplementedError('cannot update metadata while file is open for writing')
+        return self.s3.setxattr(self.path, **kwargs)
+
     def url(self):
         """ HTTP URL to read this file (if it already exists)
         """
@@ -965,7 +1068,7 @@ class S3File(object):
                 self.mpu = self.mpu or self.s3.s3.create_multipart_upload(
                                 Bucket=self.bucket, Key=self.key, ACL=self.acl)
             except (ClientError, ParamValidationError) as e:
-                raise IOError('Initating write failed: %s' % self.path, e)
+                raise IOError('Initiating write failed: %s' % self.path, e)
 
             while True:
                 try:
